*~~~~~~~~~~~~~~~~~~ 6. DATA ENTRY ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
$ontext
This file
  1.   Coverts sets and data in Excel to GDX
        GDX file labelled 'data_in.gdx'
  2.   Load sets and data (not prior SAM) from data_in.gdx
  3.   Load Prior SAM from data_in.gdx or 'gdxname.gdx'
  4.   Define sets that exclude total and other elements
  5.   Set DEFAULTS to control the model
         default_SAM    - assigned in the code
         opt_macro      - assigned in the controls worksheet
         opt_RC         - assigned in the controls worksheet
         opt_tv         - assigned in the controls worksheet
         opt_msam       - assigned in the controls worksheet
  6.   Assign parameters for Aggregate Targets
  7.   Select ERROR Support Sets
  8.   SAM scaling
         P_SAM
         MACSAM_EXOG
         MACTOTAL0
         rowcol_tot

$offtext

*~~~~~~ 1. Convert EXCEL file data to GDX ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

$CALL "GDXXRW i=1_data\%samest_data%.xlsx o=1_data\data_in.gdx INDEX=LAYOUT!A4 trace=3"
$if errorLevel 1 $abort problems with GDXXRW

*~~~~~~ 2. Load sets and data (not prior SAM) from data_in.gdx ~~~~~~~~~~~

$GDXIN 1_data\data_in.gdx

* Load Micro and Macro Sets from data_in.gdx

$LOADdc sac c m a f l k lnd h g  e i w emrgn etax
$LOADdc gtind gtdir
$LOADdc ss

* Control Sets
$LOADdc aggr d_cons m_cons 
*$LOADdc aggr2 
$LOADdc sig alt aggt esuppset esupp

* Macro SAM Data
$LOADdc macscale

* Values for controlling parameters
$LOADdc d_controls m_controls stderrs rowcol_tot stderrs_MSAM

*$LOADdc aggrtotal0  stderrs2
$LOADdc aggmat
$LOADdc sigmac cmac

*Load arbitrary SAM aggregator

$LOADdc nsam SAMOPT_TV

$GDXIN

*~~~~~~ 3. Load Prior SAM and MACRO SAM ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*~~~~~ Prior SAM
$onText
  The prior SAM can be accessed from
    1.  data_in.gdx if the SAM is in the Excel workbook
    2.  An alternative GDX file
    
 SETGLOBAL 'p_sam_in' identifies the sources

$offtext

$GDXIN 1_data\%p_sam_in%.gdx

*~~~ Load Prior SAM
$LOADdc P_SAM  = P_SAM

$GDXIN

*~~~~~ Exogenous Macro SAM

$onText
  The Macro SAM can be accessed from
    1.  data_in.gdx if the Macro SAM is in the Excel workbook
    2.  An alternative GDX file
    
 SETGLOBAL 'macsam_in' identifies the sources

$offtext

$GDXIN 1_data\%macsam_in%.gdx

*~~~ Load Prior macro-SAM
$LOADdc MACSAM_EXOG

$GDXIN

*~~~~~~ 4. Define Singleton sets ~~~~~~~~~~~~~
$onText
 Singleton sets are used in this code
     to remove quoted elements from the code - this means that the labels in the
     SAM do NOT have to be exactly those in the code,
     if not the assignments to each singleton set MUST be assigned.
 
      This model uses singleton sets for the following groups
      
 Singleton sets are NOT used for quoted elements that are loaded from
 worksheets in the Excel workbook other than the 'SETS' worksheet
       
$offtext

$INCLUDE 1_data\samest_single_set.inc

*~~~~~~ 5. Define sets that exclude total and other elements ~~~~~~~~~~~~~

 sacn(sac)          = YES ;
 sacn('total')      = NO ;
 ssn(ss)            = YES ;
 ssn("m_sam_tot")   = NO ;
 ss2n(ss)           = ssn(ss) ;
* ss2n("fctr")      = NO ;

* ss3n(ssn)          = YES ;

*~~~~~~ 6. Set DEFAULTS to control the model ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

* Set options to manage settings. default_SAM is set in code.
 opt_blocks       = m_controls("con_opt_blocks","base") ;
 opt_agg          = m_controls("con_opt_agg","base") ;
 opt_RC_exog      = m_controls("con_opt_RC_exog","base") ;
 opt_tv           = m_controls("con_opt_tv","base") ;
 opt_msam         = m_controls("con_opt_msam","base") ;
 
 opt_blocks${(opt_blocks ne 0.0) and (opt_blocks ne 1.0)}    = 0.0 ;
 opt_AGG${(opt_agg ne 0.0) and (opt_agg ne 1.0)}             = 0.0 ;
 opt_RC_exog${(opt_RC_exog ne 0.0) and (opt_RC_exog ne 1.0)} = 0.0 ;
 opt_tv${(opt_tv ne 0.0) and (opt_tv ne 1.0)}                = 0.0 ;
 opt_msam${(opt_msam ne 0.0) and (opt_msam ne 1.0)}          = 0.0 ;

* If opt_macro = 0, set sigmac and cmac to zero
 SIGMAC(ss,ssp)$(opt_blocks ne 1)    = 0 ;
 CMAC(ss,ssp)$(opt_blocks ne 1)      = 0 ;

*~~~~~~ 7. Assign parameters for Aggregate Targets ~~~~~~~~~~~~~~~~~~~~~~~

* Set aggregate control totals to use if opt_agg = 0 or 1

* Assign members of aggr2 based on AGGMAT

* aggr2(aggr)$[aggmat(aggr,"aggr_inc") NE = 0.0]  = YES ;
 aggr2(aggr)$[aggmat(aggr,"aggr_inc") = 1]  = YES ;

 aggtotal0(aggr2)           = aggmat(aggr2,"value_AGG") ;
 stderrs_AGG(aggr2)         = aggmat(aggr2,"stdERR_AGG") ;


* load stderrs_RC(sac) from rowcol1 if opt_RC_exog = 0 and rowcol_tot(sac,"stderr_RC") NE 0

IF( opt_RC_exog eq 0,
IF{
    SUM(sac,rowcol_tot(sac,"stderr_RC")) NE 0.0 ,
     stderrs_RC(sac)  = rowcol_tot(sac,"stderr_RC")  ;
        
ELSE
    stderrs_RC(sac) = stderrs("sig_RC","alt0") ;
   } ;
   
 ) ;
 
IF( opt_RC_exog eq 1,
    stderrs_RC(sac) = stderrs("sig_RC","alt0") ;
    
) ;

$ontext
   Set specific cell values of SAM if opt_tv = 1 and there are values
 in the parameter SAMOPT_TV(sacn,sacnp,"value_tv").
$offtext 

 P_SAM(sacn,sacnp)${(opt_tv eq 1) and SAMOPT_TV(sacn,sacnp,"value_tv")} =
                    SAMOPT_TV(sacn,sacnp,"value_tv") ;

*~~~ Select totals for CONSISTENCY

 target           = m_controls("con_target","base") ;

*~~~~~~ 8. Select ERROR Support Sets ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
$onText
 Add options to allow 3, 5, and 7 element support sets for
 various parts of the SAM. The default in errsupp is used for all parts
 of the SAM if an alternative is not read in the Excel worksheet.
$offtext

*~~~ Select standard errors

 sigm             = m_controls("con_sigm","base") ;
 
*~~~~~~ 9. SAM scaling ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
$ontext

   Scaling proto-sam (P_SAM) and macro-sam (MACSAM_EXOG) for improved
clarity and algorithm performance

AUTOMATIC CALCULATING OF A SCALING FACTOR - samscal

   This instruction works by successively increasing the samscal parameter by
multiples of 10 WHILE a given proportion - d_controls("scalprop","base" -
of the SAM entries are below a given target - d_controls("scaltarg","base").

   The conditional statement is computed as follows
Denominator - the total number of non zero cells in the SAM(sacn,sacnp)
Numerator - the total number of cells in SAM(sacn,sacnp) that are
            i)  non zero cells; AND
            ii) whose absolute values when divided by the current value of
                samscal are less that the target.

   Initial values set in Excel when parameters were defined:
 samscal0        Initial SAM scaling factor    /.001/
 scaltarg        SAM autoscaling target        /100/
 scalprop        SAM autoscaling proportion    /0.975/
 
   Experience indicates that scalprop between 0.925 and 0.975
for scaltarg of 100 is the appropriate order of magnitude.

   But algorithm performance (Major and Minor interations and OB) should
be monitored when first solving with 'wide' error bounds

The ABORT statement protects against illegal values of samautoscaltarg

$offtext

*~~~~ Assign cutoff value for small cells

 cutoff0          = d_controls("cutoff0","base") ;
 
* Assigning values to initialise scaling parameter

*~~~~~~ Assign parameters for Autoscaling of the SAM

 samscal          = d_controls("samscal","base") ;

$ontext
 Assigning values to programme control parameters
 This assigns samscal for autoscaling scaling.
 It is overwritten is if manual scaling is chosen,
$offtext 

 samautoscal      = d_controls("samautoscal","base") ;

 
*~~~~~~ SAM Scaling

 ABORT $(d_controls("scaltarg","base") LE 0.0)
 "samautoscaltarg is less than or equal to ZERO - set to positive value" ;
 
WHILE{
 ( {SUM[(sacn,sacnp)$(P_SAM(sacn,sacnp)
            AND {ABS[P_SAM(sacn,sacnp)/samscal ]
              < d_controls("scaltarg","base") } ), 1.0 ] }
        /SUM[(sacn,sacnp)$P_SAM(sacn,sacnp), 1.0 ]) < d_controls("scalprop","base") ,

*     samscal is increased in multiples of 10 until the condition is satisfied
      samscal = samscal * 10;
  } ;

*~~ Select manual scaling
$onText
If samautoscal eq 1, then samscal is set by autoscaling, else manual
using d_controls("samscalman","base") as the scaling factor
$offtext

 samscal$(samautoscal eq 0.0 ) = d_controls("samscalman","base") ;
 
* Check for mistake in read in value of samscal

 ABORT $(samscal LE 0.0)
 "samautoscaltarg is less than or equal to ZERO - set to positive value" ;
 
*~~~ Implement scaling
 
* SAM(sacn,sacnp) = SAM(sacn,sacnp)/samscal ;
 P_SAM(sac,sacp)             = P_SAM(sac,sacp)/samscal ;
 P_SAM0(sac,sacp)            = P_SAM(sac,sacp) ;

* Assume that MACSAM_EXOG is already scaled if macscale=1.
 macsam_exog(ss,ssp)$(macscale eq 0)    = macsam_exog(ss,ssp)/samscal ;
 
* Scaling row and column totals

 rowcol_tot(sac,"rowsum1")    = rowcol_tot(sac,"rowsum1")/samscal ;
 rowcol_tot(sac,"colsum1")    = rowcol_tot(sac,"colsum1")/samscal ;
 
* Scaling Other aggregates

 AGGTOTAL0(aggr)              = AGGTOTAL0(aggr)/samscal ;
 
 cutoff0                      = cutoff0/samscal ;
 
*~~~~ Testing scalings are consistent
 
Parameter
 test_scaling1     Test for scaling on P-SAM and MACSAM_EXOG
 test_scaling2     Test for scaling on P-SAM and rowcol_tot
 
 ;
 
 test_scaling1$macsam_exog("cmdty","actvty")
               = macsam_exog("cmdty","actvty")/SUM[(c,a),P_SAM(c,a)] ;
 
 test_scaling2$SUM[c,rowcol_tot(c,"rowsum1")]
                = SUM[c,rowcol_tot(c,"rowsum1")]/SUM[(c,sacn),P_SAM(c,sacn)] ;
 
 ABORT$(test_scaling1 GT 8.0)
 "Scalings for P_SAM and MACSAM_EXOG are inconsistent. Check P_SAM and MACSAM_EXOG values" ;
 
 ABORT$(test_scaling2 GT 8.0)
 "Scalings for P_SAM and rowcol_tot are inconsistent. Check P_SAM and rowcol_tot values" ;
 
* Initialize row and column sums to zero

 P_SAM(t_tot,sacp) = 0.0 ;
 P_SAM(sac,t_tot)  = 0.0 ;

*~~~~~~~~~~~~~~~~~~ END OF DATA ENTRY ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~