*~~~~~~~~~ 7e. FORCE CONSISTENCY FOR CONSTRAINT & ERROR SPECIFICATIONS ~~~

$ontext
 This file checks for consistency in the specifications for cells

  There are 2 parts each with several steps 
 1.  Ensure cells NOT specified as values (ivalue) are specified as
     coefficients.
     a.  CHK ivalue has a NON zero column total (icolnz)
     b.  CHK ivalue is NEGATIVE (ineg)
     c.  REMOVE ivalue if all entries are zero (izero)
     d.  Set icoeff if NOY ivalue and NOT ZERO
     e.  Ensure all ivalue and icoeff are estimated
     f.  Assign ifixv as cells not estimated
     g.  Set additive errors for cells that are NEGATIVE and ESTIMATED
     h.  Set multiplicative error for cells ESTIMATED without ADDITIVE
     
  2. Check for errors in setting of constraints. These errors cause
     the programme to ABORT with error message
     a.  Cells with both ivalue and icoeff
     b.  Cells to be estimated but neither acell nor lcell
     c.  Cells to be estimated but both acell and lcell
     d.  Cells to be estimated but initially zero
     e.  Accounts in irow or icol but not in sacn

$offtext


*~~~~~~ ENSURE CELLS NOT SPECIFIED AS VALUES ARE COEFFICIENTS ~~~~~~~~~~~~

*Cells which are not specified as values will be specified as coefficients.
*Ensure that negative cells are specified as values and zero cells are omitted.
* If column sum is zero, estimate cell values.

 ivalue(irow,icol)$(not icolnz(icol))           = YES ;
 ivalue(irow,icol)$ineg(irow,icol)              = YES ;
 ivalue(irow,icol)$izero(irow,icol)             = NO ;

 icoeff(irow,icol)$(not ivalue(irow,icol) and (not izero(irow,icol))) = YES ;

 estimate(irow,icol)$(ivalue(irow,icol) or icoeff(irow,icol)) = YES ;

 ifixv(irow,icol)$(not estimate(irow,icol))     = YES ;

*Make sure that acell is on for negative cells
 acell(irow,icol)$(ineg(irow,icol) and estimate(irow,icol))        = YES ;

*All cells that are not acell will be lcell, with multiplicative errors
 lcell(irow,icol)$((NOT acell(irow,icol)) and estimate(irow,icol)) = YES ;
 
*~~~~~~ CHECK FOR ERRORS IN SETTING OF CONSTRAINTS ~~~~~~~~~~~~~~~~~~~~~~~

* Cells cannot be constrained in both values and coefficients.
* All cells to be estimated must have lcell or acell.

 icheck(irow,icol)$(ivalue(irow,icol) and icoeff(irow,icol)) = YES ;
 abort$(SUM(icheck,1))
   "Error check. Cells with both ivalue and icoeff", icheck ;

 icheck(irow,icol)$(estimate(irow,icol) and ((not acell(irow,icol)) and
                                                not lcell(irow,icol))) = YES ;
 abort$(SUM(icheck,1))
   "Error check. Cells to be estimated but neither acell nor lcell", icheck ;

 icheck(irow,icol)$(estimate(irow,icol) and (acell(irow,icol) and
                                                lcell(irow,icol))) = YES ;
 abort$(SUM(icheck,1))
   "Error check. Cells to be estimated but both acell and lcell", icheck ;

 icheck(irow,icol)$(estimate(irow,icol) and izero(irow,icol)) = YES ;
 abort$(SUM(icheck,1))
   "Error check. Cells to be estimated but initially zero", icheck ;

* The description below differed from the icheck2 definition and neither
* matched the code below (which resulted in an abort)

* icheck2(sacn)$((not irow(sacn)) or (not icol(sacn))) = YES ;

* suggested replacement code (icheck2 parameter definition also updated
* above). Basically it checks that the totals row/cols are not included in the
* estimate

 icheck2(sac) = ((NOT sacn(sac)) AND (irow(sac) OR icol(sac)));
 abort$(SUM(icheck2,1))
   "Error check. Accounts in irow or icol but not in sacn", icheck2 ;
   
*~~~~~~~~~ END OF FORCE CONSISTENCY FOR CONSTRAINT & ERROR SPECIFICATIONS