*~~~~~~~~~~~~~~ 8. SET ERROR BOUNDS AND SUPPORT SETS ~~~~~~~~~~~~

$ontext
 Assume a prior on standard deviation of the errors, sigma.

 Start from assumed prior knowledge of the standard error (perhaps due
 to measurement error) of the column sums. Below, we assume that all
 column sums have a standard error of 5%, or, alternatively, a user-defined
 value. This is a Bayesian prior, not a maintaned hypothesis.
 The estimated error is weighted sum of elements in an error support
 set, e.g.,
  ERR_**(**,**) =E= SUM(jwt_**, W_**(**,**,jwt_**)*vbar_**(**,**,jwt_**V)) ; 
 where the W_** are estimated in the CE procedure.
 
The prior variance of these errors is given by:
   (sigmay(sacn))**2 = SUM(jwt, WBAR(sacn,jwt)*(VBAR(sacn,jwt))**2 )
 where the WBAR's are the prior on the weights.
 The VBARs are chosen to define a domain for the support set of +/- 3
 standard errors. The prior on the weights, WBAR, are then calculated
 to yield the specified prior on the standard error, sigmay.
 In Robinson, Cattaneo, and El-Said (2001), we specify prior weights
 (WBAR) that are uniform and set the prior standard error by the
 choice of support set, VBAR. In that paper, we use a three-weight
 specification (jwt /1*3/);

 We define three sets of errors with separate weights, W1, W2, and W_TV. The
 first is for specifying errors on column sums, the second for errors
 on macro aggregates (defined in the set macro), and the third for errors
 in cell values.

 If sigmay is set to zero, the corresponding constraint is assumed to be
 exact, with no error.

 Define standard errors for errors on column sums, sigmay_RC. In this version,
 if the initial column sum is zero, then sigmay_AGG will also be zero and the
 account will be constrained to sum to zero. This is appropriate for a
 "balancing" account. However, a nonzero sum with or without error can also
 be specified.

 Set sigmay_RC and target column sums

$offtext

* --- jwt - Error Support Sets ----------------------------------------------

*~~~~~ CHECK that the number of elements in esupp is correct and of 'allowed' values
 
 LOOP{esuppset,
  IF[{(esupp(esuppset) ne 3) AND
      (esupp(esuppset) ne 5) AND
      (esupp(esuppset) ne 7)},
   display "#*#*# ERROR CHECK #*#*# Error in esupp #*#*#", esupp ;
   esupp(esuppset) = 0 ;
*End if
   ] ;
*End loop
   } ;
   
Parameter
 esupp_test    Test that esupp assigned for all esuppset
  ;
   
 esupp_test = SUM[esuppset$(esupp(esuppset) EQ 0) , 1 ] ;
 
 ABORT$(esupp_test GT 0.0)
 "Error support sets not set for at least one element. Check esupp in Excel", esupp_test ;
   
$ontext

 LOOP{esuppset,
  IF[{(esupp(esuppset) ne 3) and
      (esupp(esuppset) ne 5) and
      (esupp(esuppset) ne 7)},
   display "#*#*# ERROR CHECK #*#*# Error in esupp #*#*#", esupp ;
*   esupp(esuppset) = 3 ;
*End if
   ] ;
*End loop
   } ;
   
$offtext

 jwt_RC(jwt)$(ord(jwt) le esupp("esupp_RC"))     = yes ;
 jwt_AGG(jwt)$(ord(jwt) le esupp("esupp_AGG"))   = yes ;
 jwt_TV(jwt)$(ord(jwt) le esupp("esupp_TV"))     = yes ;
 jwt_MSAM(jwt)$(ord(jwt) le esupp("esupp_MSAM")) = yes ;
 

* --- Define standard errors for errors ----------------------------------

* First determine default sigcoefs. These are initially set using the
* same value for all accounts.

* This option chooses a default setting for the Standard Errors for Errors
 IF(((sigm ne 0) and (sigm ne 1) and (sigm ne 2) ),
  Display "### ERROR CHECK ### invalid value of sigm ###", sigm;
  sigm = 0 ;
 ) ;

if(sigm = 0,
  sig_RC_coef(sacn,rwcl)                    = 0.25 ;
  sig_AGG_coef(aggr2)                       = 0.05 ;
  sig_TV_coef(sac,sacp)$nonzero(sac,sacp)   = 0.25 ;
* sig4coef(ssn,ssnp) = 0.05 ;
* stderrs4(ssn,ssnp)$stderrs4(ssn,ssnp) = 0.05
   ) ;

* Otherwise, use one of option sigm = 1 or sigm = 2 for user-defined defaults

if(sigm = 1,
  sig_RC_coef(sacn,rwcl)                    = stderrs("sig_RC","alt1") ;
  sig_AGG_coef(aggr2)                       = stderrs("sig_AGG","alt1") ;
  sig_TV_coef(sac,sacp)$nonzero(sac,sacp)   = stderrs("sig_TV","alt1") ;
* stderrs4(ssn,ssnp)$stderrs4(ssn,ssnp) = stderrs("sig4","alt1") ;
* sig4coef(ssn,ssnp) = stderrs("sig4","alt1") ;
   ) ;

if(sigm = 2,
  sig_RC_coef(sacn,rwcl)                    = stderrs("sig_RC","alt2") ;
  sig_AGG_coef(aggr2)                      = stderrs("sig_AGG","alt2") ;
  sig_TV_coef(sac,sacp)$nonzero(sac,sacp)   = stderrs("sig_TV","alt2") ;
* stderrs4(ssn,ssnp)$stderrs4(ssn,ssnp) = stderrs("sig4","alt2") ;
* sig4coef(ssn,ssnp) = stderrs("sig4","alt1") ;
   ) ;

if(opt_blocks = 1,
* Start with sigm value set above
* Start with alt2 default
* sig_RC_coef(sacn,rwcl)                = stderrs("sig_RC","alt2") ;
*If there are entries in sig1mac(sac), use them
  sig_RC_coef(sacn,rwcl)$sig1mac(sacn)        = sig1mac(sacn) ;
*If there are values in stderrs1(sacn), use them
  sig_RC_coef(sacn,rwcl)$stderrs_RC(sacn)     = stderrs_RC(sacn) ;
*Replace negative values with zero
  sig_RC_coef(sacn,rwcl)$(sig_RC_coef(sacn,rwcl) lt 0) = 0 ;

* sig_AGG_coef(aggr2)                   = stderrs("sig_AGG","alt2") ;
*If there are values in stderrs2(aggr2), use them
* sig_AGG_coef(aggr2)$stderrs2(aggr2)  = stderrs2(aggr2) ;

* sig_TV_coef(sac,sacp)                   = stderrs("sig3","alt2") ;
* Take account of standard errors read in sigmac(ss,ssp) for blocks
  sig_TV_coef(sacn,sacnp)$(sig3mac(sacn,sacnp) gt 0)  = sig3mac(sacn,sacnp) ;
  sig_TV_coef(sacn,sacnp)$(sig3mac(sacn,sacnp) lt 0)  = 0 ;
* sig4coef(ssn,ssnp) = stderrs("sig4","alt2") ;
   ) ;

* Note that if opt_msam = 0, then aggr2 is empty.
* sig_AGG_coef(aggr2)                   = stderrs("sig_AGG","alt2") ;
*If there are values in stderrs2(aggr2), use them
  sig_AGG_coef(aggr2)$stderrs_AGG(aggr2)  = stderrs_AGG(aggr2) ;
* Set to zero if a negative number is read in macromat
  sig_AGG_coef(aggr2)$(stderrs_AGG(aggr2) lt 0)  = 0 ;

* Set entries in macset as included if there is an entry in stderrs(ss,ss)
* The value of sig_MAM_coef will be set to the value in stderrs_MSAM. If negative,
* however, the value is set to zero. So, a negative entry implies that the
* cell is included in the constraint set, but that sigmay_MSAM will be zero.
* Membership in set MACSET is determined in mactotal_eg_agg.inc.

* set sig_MSAM_coef. Must be defined but is only used if opt_AGGro = 1
* Redo to allow column and row totals to be used as macro constraints
*if(opt_macro = 1,

  sig_MSAM_coef(ss,ssp)$macset(ss,ssp)          = stderrs_MSAM(ss,ssp) ;
  sig_MSAM_coef(ss,ssp)$(sig_MSAM_coef(ss,ssp) lt 0) = 0 ;
*  ) ;

* Set sigmay values

* Set sigmay_RC on row and column sums
  sigmay_RC(sacn,"col1")$(icol2(sacn) and acol(sacn))
                       = sig_RC_coef(sacn,"col1") * ABS(coltarget(sacn)) ;

  sigmay_RC(irow,"row1")$irow2(irow)
                       = sig_RC_coef(irow,"row1") * ABS(rowtarget(irow)) ;

* Set sigmay_AGG, standard errors for errors on macro aggregates.
  sigmay_AGG(aggr2)  = sig_AGG_coef(aggr2) * ABS(aggtotal(aggr2)) ;

$ontext
 Set errors on cell elements, sigmay_TV. They can be on values or coefficients,
 and can be specified as additive or multiplicative. If multiplicative, the
 prior is implicitly specified as lognormal. That is, log(error) is
 distributed as a normal with mean zero and standard deviation sigmay_TV.
 Then EXP(error), which is the multiplicative error, has mean one.

 Set sigmay_TV, standard errors for errors on cell values.
 In default case, lcell and icoeff will be empty.
$offtext

* Use values from SAMOPT_TV, if opt_tv = 1
 IF(opt_tv eq 1,
  sig_TV_coef(irow,icol)$SAMOPT_TV(irow,icol,"stderr_tv")
            = SAMOPT_TV(irow,icol,"stderr_tv");

  sig_TV_coef(irow,icol)$(sig_TV_coef(irow,icol) lt 0) = 0 ;
*End if
 ) ;

  sigmay_TV(irow,icol)$(ivalue(irow,icol) and acell(irow,icol))
    = sig_TV_coef(irow,icol)*ABS(P_SAM(irow, icol))   ;

  sigmay_TV(irow,icol)$(ivalue(irow,icol) and lcell(irow,icol))
    = sig_TV_coef(irow,icol)   ;

  sigmay_TV(irow,icol)$(icoeff(irow,icol) and acell(irow,icol))
    = sig_TV_coef(irow,icol)*ABS(coeff0(irow,icol))   ;

  sigmay_TV(irow,icol)$(icoeff(irow,icol) and lcell(irow,icol))
    = sig_TV_coef(irow,icol)   ;

$ontext
 Set errors for macro-sam aggregates, sigmay_MSAM. These are assumed to be
 additive.
$offtext
 display "macsam0 check", macsam0 ;

  sigmay_MSAM(ss,ssp)$macset(ss,ssp)
    = sig_MSAM_coef(ss,ssp)*ABS(MACSAM0(ss,ssp)) ;

* --- Define support sets ------------------------------------------------
* Add 7-element support set and new specification of 5-element set

 IF((card(jwt_RC) eq 3),
$ontext
3 weights
 This code assumes a prior mean of zero and a two-parameter
 distribution with specified prior standard error. There are three
 weights, W(sacn,jwt), to be estimated. The actual moments are estimated
 as part of the estimation procedure.
$offtext

* Set constants for 3-weight error distribution
  vbar_RC(sacn,rwcl,"1")  = -3 * sigmay_RC(sacn,rwcl);
  vbar_RC(sacn,rwcl,"2")  =  0             ;
  vbar_RC(sacn,rwcl,"3")  = +3 * sigmay_RC(sacn,rwcl);

  wbar_RC(sacn,rwcl,"1")  =  1/18  ;
  wbar_RC(icol,rwcl,"2")  = 16/18  ;
  wbar_RC(icol,rwcl,"3")  =  1/18  ;
*End if
 ) ;


$ontext
 5 weights
 This code assumes a prior mean of zero, no skewness (symmetric distribution),
 and a prior value of kurtosis consistent with a prior normal distribution
 with mean zero, variance sigmay**2, and kurtosis equal to 3*sigmay**4.
 The addition of a prior on skewness and kurtosis requires estimation
 of 5 weights (jwt = 5). The prior weights wbar are specified so that:
 SUM(jwt, wbar(sacn,jwt)*vbar(sacn,jwt)**4) = 3*sigmay(sacn,jwt)**4
 as well as defining the variance as above.
 The prior weights and support set are also symmetric, so the prior
 on all odd moments is zero (no skewness). The choice of +/- 1 standard error
 for vbar(sacn,"2") and vbar(sacn,"4") is arbitrary.
 The actual moments are estimated as part of the estimation procedure.
$offtext

 IF((card(jwt_RC) eq 5),
* Set constants for 5-weight error distribution
  vbar_RC(sacn,rwcl,"1")    = -3   * sigmay_RC(sacn,rwcl) ;
  vbar_RC(sacn,rwcl,"2")    = -1.5 * sigmay_RC(sacn,rwcl) ;
  vbar_RC(sacn,rwcl,"3")    =  0 ;
  vbar_RC(sacn,rwcl,"4")    = +1.5 * sigmay_RC(sacn,rwcl) ;
  vbar_RC(sacn,rwcl,"5")    = +3   * sigmay_RC(sacn,rwcl) ;

  wbar_RC(sacn,rwcl,"1")    =  1/162 ;
  wbar_RC(sacn,rwcl,"2")    =  16/81  ;
  wbar_RC(sacn,rwcl,"3")    =  48/81  ;
  wbar_RC(sacn,rwcl,"4")    =  16/81  ;
  wbar_RC(sacn,rwcl,"5")    =  1/162 ;
*end if
 ) ;
 
$onText
7 weights
 This is an 'uninformative' prior that incorporates only information about
 the outer bounds between which the errors must fall. In Bayesian analysis,
 the continuous uninformative prior is the uniform distribution between
 the bounds. Assuming the bounds are specified as + or - 3s, where s is
 a constant, then the prior mean is zero and the variance of the
 continuous uniform distribution can be calculated.

 The decision is here is to specify a finite distribution with an
 evenly-spaced, seven-element support set, with uniform prior weights
 all equal to 1/7, for which the variance can be calculated.
    
 The seven-element finite prior distribution provides a conservative
 specification of an uninformative prior. Adding more elements would
 more closely approximate the continuous uniform distribution. Using
 fewer elements yields a higher prior variance. The seven-element
 specification permits the estimated posterior distribution to be
 essentially unconstrained. In this case, with limited data, it will
 not be possible to recover much information about the posterior
 distribution. 

$offtext


 IF((card(jwt_RC) eq 7),
* Set constants for 7-weight error distribution
  vbar_RC(sacn,rwcl,"1")    = -3 * sigmay_RC(sacn,rwcl) ;
  vbar_RC(sacn,rwcl,"2")    = -2 * sigmay_RC(sacn,rwcl) ;
  vbar_RC(sacn,rwcl,"3")    = -1 * sigmay_RC(sacn,rwcl) ;
  vbar_RC(sacn,rwcl,"4")    =  0 ;
  vbar_RC(sacn,rwcl,"5")    = +1 * sigmay_RC(sacn,rwcl) ;
  vbar_RC(sacn,rwcl,"6")    = +2 * sigmay_RC(sacn,rwcl) ;
  vbar_RC(sacn,rwcl,"7")    = +3 * sigmay_RC(sacn,rwcl) ;

  wbar_RC(sacn,rwcl,jwt_RC)   = 1/7 ;
*end if
 ) ;

 IF((card(jwt_AGG) eq 3),
* Set constants for 3-weight error distribution
  vbar_AGG(aggr2,"1")  = -3 * sigmay_AGG(aggr2);
  vbar_AGG(aggr2,"2")  =  0 ;
  vbar_AGG(aggr2,"3")  = +3 * sigmay_AGG(aggr2);

  wbar_AGG(aggr2,"1")  =  1/18 ;
  wbar_AGG(aggr2,"2")  = 16/18 ;
  wbar_AGG(aggr2,"3")  =  1/18 ;
*End if
 );

 IF((card(jwt_AGG) eq 5),
* Set constants for 5-weight error distribution
  vbar_AGG(aggr2,"1")  = -3   * sigmay_AGG(aggr2) ;
  vbar_AGG(aggr2,"2")  = -1.5 * sigmay_AGG(aggr2) ;
  vbar_AGG(aggr2,"3")  =  0 ;
  vbar_AGG(aggr2,"4")  = +1.5 * sigmay_AGG(aggr2) ;
  vbar_AGG(aggr2,"5")  = +3   * sigmay_AGG(aggr2) ;

  wbar_AGG(aggr2,"1")  =   1/162 ;
  wbar_AGG(aggr2,"2")  =  16/81  ;
  wbar_AGG(aggr2,"3")  =  48/81  ;
  wbar_AGG(aggr2,"4")  =  16/81  ;
  wbar_AGG(aggr2,"5")  =   1/162 ;
*end if-else
 );

 IF((card(jwt_AGG) eq 7),
* Set constants for 7-weight error distribution
  vbar_AGG(aggr2,"1")  = -3 * sigmay_AGG(aggr2) ;
  vbar_AGG(aggr2,"2")  = -2 * sigmay_AGG(aggr2) ;
  vbar_AGG(aggr2,"3")  = -1 * sigmay_AGG(aggr2) ;
  vbar_AGG(aggr2,"4")  =  0 ;
  vbar_AGG(aggr2,"5")  = +1 * sigmay_AGG(aggr2) ;
  vbar_AGG(aggr2,"6")  = +2 * sigmay_AGG(aggr2) ;
  vbar_AGG(aggr2,"7")  = +3 * sigmay_AGG(aggr2) ;

  wbar_AGG(aggr2,jwt_AGG) = 1/7 ;

*end if-else
 );
 IF((card(jwt_TV) eq 3),
* Set constants for 3-weight error distribution
  vbar_TV(irow,icol,"1")  = -3 * sigmay_TV(irow,icol);
  vbar_TV(irow,icol,"2")  =  0             ;
  vbar_TV(irow,icol,"3")  = +3 * sigmay_TV(irow,icol);

  wbar_TV(irow,icol,"1")  =  1/18  ;
  wbar_TV(irow,icol,"2")  = 16/18  ;
  wbar_TV(irow,icol,"3")  =  1/18  ;
*End if
 ) ;

 IF((card(jwt_TV) eq 5),
* Set constants for 5-weight error distribution
  vbar_TV(irow,icol,"1")  = -3   * sigmay_TV(irow,icol);
  vbar_TV(irow,icol,"2")  = -1.5 * sigmay_TV(irow,icol);
  vbar_TV(irow,icol,"3")  =  0                  ;
  vbar_TV(irow,icol,"4")  = +1.5 * sigmay_TV(irow,icol);
  vbar_TV(irow,icol,"5")  = +3   * sigmay_TV(irow,icol);

  wbar_TV(irow,icol,"1")  =  1/162 ;
  wbar_TV(irow,icol,"2")  =  16/81  ;
  wbar_TV(irow,icol,"3")  =  48/81  ;
  wbar_TV(irow,icol,"4")  =  16/81  ;
  wbar_TV(irow,icol,"5")  =  1/162 ;
*End if
 ) ;

 IF((card(jwt_TV) eq 7),
* Set constants for 7-weight error distribution
  vbar_TV(irow,icol,"1")  = -3 * sigmay_TV(irow,icol);
  vbar_TV(irow,icol,"2")  = -2 * sigmay_TV(irow,icol);
  vbar_TV(irow,icol,"3")  = -1 * sigmay_TV(irow,icol);
  vbar_TV(irow,icol,"4")  =  0         ;
  vbar_TV(irow,icol,"5")  = +1 * sigmay_TV(irow,icol);
  vbar_TV(irow,icol,"6")  = +2 * sigmay_TV(irow,icol);
  vbar_TV(irow,icol,"7")  = +3 * sigmay_TV(irow,icol);

  wbar_TV(irow,icol,jwt_TV) = 1/7        ;

*End if
 ) ;

*SR 04/06 Adjust to allow ss instead of ssn

 IF((card(jwt_MSAM) eq 3),
* Set constants for 3-weight error distribution
  vbar_MSAM(ss,ssp,"1")  = -3 * sigmay_MSAM(ss,ssp);
  vbar_MSAM(ss,ssp,"2")  =  0             ;
  vbar_MSAM(ss,ssp,"3")  = +3 * sigmay_MSAM(ss,ssp);

  wbar_MSAM(ss,ssp,"1")  =  1/18  ;
  wbar_MSAM(ss,ssp,"2")  = 16/18  ;
  wbar_MSAM(ss,ssp,"3")  =  1/18  ;
*End if
 );

 IF((card(jwt_MSAM) eq 5),
* Set constants for 5-weight error distribution
  vbar_MSAM(ss,ssp,"1")  = -3   * sigmay_MSAM(ss,ssp);
  vbar_MSAM(ss,ssp,"2")  = -1.5 * sigmay_MSAM(ss,ssp);
  vbar_MSAM(ss,ssp,"3")  =  0                ;
  vbar_MSAM(ss,ssp,"4")  = +1.5 * sigmay_MSAM(ss,ssp);
  vbar_MSAM(ss,ssp,"5")  = +3   * sigmay_MSAM(ss,ssp);

  wbar_MSAM(ss,ssp,"1")  =  1/162 ;
  wbar_MSAM(ss,ssp,"2")  =  16/81  ;
  wbar_MSAM(ss,ssp,"3")  =  48/81  ;
  wbar_MSAM(ss,ssp,"4")  =  16/81  ;
  wbar_MSAM(ss,ssp,"5")  =  1/162 ;
*End if
 ) ;

 IF((card(jwt_MSAM) eq 7),
* Set constants for 5-weight error distribution
  vbar_MSAM(ss,ssp,"1")  = -3 * sigmay_MSAM(ss,ssp);
  vbar_MSAM(ss,ssp,"2")  = -2 * sigmay_MSAM(ss,ssp);
  vbar_MSAM(ss,ssp,"3")  = -1 * sigmay_MSAM(ss,ssp);
  vbar_MSAM(ss,ssp,"4")  =  0          ;
  vbar_MSAM(ss,ssp,"5")  = +1 * sigmay_MSAM(ss,ssp);
  vbar_MSAM(ss,ssp,"6")  = +2 * sigmay_MSAM(ss,ssp);
  vbar_MSAM(ss,ssp,"7")  = +3 * sigmay_MSAM(ss,ssp);

  wbar_MSAM(ss,ssp,jwt_MSAM) = 1/7    ;
*End if
 ) ;


*~~~~~~~~~~~~~~~~ END OF SET ERROR BOUNDS AND SUPPORT SETS ~~~~~~