*~~~~~~~~~~~~~~~~~ DEFINE MACSAM_ENDOG ~~~~~~~~~~~~~~~~~

$ontext
 Include file produces an aggregated macro SAM, MACSAM_ENDOG, from the
 prior SAM, P_SAM.
 
 The aggregation is controlled by the set map_ss_sac, which is defined
 from subsets in the Excel file. Sets are defined in the main program
 and loaded from excel.
 
 Specification of map_ss_sac below must be changed if the aggregates
 in the excel file are changed.
 
 The parameters used are defined in the main program.

 The set ss is loaded from Excel worksheet 'SETS'
 Ensure ss is consistent with any Exogenous MACSAM_EXOG
 
 Define aggregates
 Micro-sam accounts may appear in more than one macro-sam aggregate account.
 Macro-sam aggregates are defined in the excel file.
 No attempt is made to impose sum constraints in macro-sam, since that is
 handled in the micro-sam.
 
 All micro-sam accounts should be listed in one macro SAM account.
 (If not a residual macro-sam account can be created).

$offtext
Set

 map_ss_sac(ss,sac)  aggregator matrix
 ;
 alias (ssn, ssnp)
 alias (ss, acmp) ;


*~~~~ Commodities Margins and activities

 map_ss_sac("cmdty",sac)$c(sac)            = YES ;
 map_ss_sac("mrgn",sac)$m(sac)             = YES ;
 map_ss_sac("actvty",sac)$a(sac)           = YES ;
 
*May combine margins with commodities.
*map_ss_sac("cmdty",sac)$(c(sac) or m(sac)) = YES ;

*~~~~ Factors
$ontext
 Macro-sam (macsam) may include either factor aggregate or separate factors,
 but not both. If there is any entry in the "fctr" row or column, assume
 that it is the control total to use, and use only aggregate of factors in
 macsam.
 Test by including both fctr and separate factor accoounts
$offtext

 IF{SUM(ssn, macsam_exog(ssn,"fmix") + macsam_exog("fmix",ssn)),
   map_ss_sac("fmix",sac)$f(sac)           = YES ;

 ELSE
   map_ss_sac("lbr",sac)$l(sac)            = YES ;
   map_ss_sac("kptl",sac)$k(sac)           = YES ;
   map_ss_sac("land",sac)$lnd(sac)         = YES ;
*end if
   } ;

*~~~~ Institutions

 map_ss_sac("hhld",sac)$h(sac)             = YES ;
 map_ss_sac("entrp",sac)$e(sac)            = YES ;
 map_ss_sac("govtn",sac)$g(sac)            = YES ;
 map_ss_sac("itax",sac)$gtind(sac)         = YES ;
 map_ss_sac("dtax",sac)$gtdir(sac)         = YES ;
 map_ss_sac("invst",sac)$i(sac)            = YES ;
 map_ss_sac("wrld",sac)$w(sac)             = YES ;

 display map_ss_sac ;

$onText
 Check map_ss_sac to ensure that each micro-sam account in sac appears in
 only one macro-sam account, ss.
$offtext

Parameter

 macmapcount(sac)      Count of sac entries in map_ss_sac 
 macmapcount_CHK(sac)  CHECK on macmapcount 0 IF correct
 ;
 
macmapcount(sacn) = SUM(ss$map_ss_sac(ss,sacn), 1) ;

macmapcount_CHK(sacn)  = macmapcount(sacn) - 1.0 ;

 display "All macmapcount entries should be 1", macmapcount ;

 display$(SUM(sacn, macmapcount(sacn)) ne CARD(sacn))
          "#### Error Check in macmapcount#####" ;
          
 ABORT${SUM(sacn,macmapcount_CHK(sacn)) GT 0.0 }
   "Error check - map_ss_sac incorrect", macmapcount ;
          
*~~~~~~~# Create Aggregate Macro-SAM - macsam_endog
$ontext
* Create aggregate macsam_endog by aggregating the proto-sam, SAM.
$offtext

 macsam_endog(ssn,ssnp) =
  SUM((sac,sacp)$(map_ss_sac(ssn,sac) and map_ss_sac(ssnp,sacp)),
        P_SAM(sac,sacp)) ;

*Computing and checking totals
 macsam_endog('m_sam_tot',ssn) = SUM(ss2np, macsam_endog(ss2np,ssn));
 macsam_endog(ssn,'m_sam_tot') = SUM(ss2np, macsam_endog(ssn,ss2np));

 macsam_bal_CHK(ssn) = macsam_endog('m_sam_tot',ssn) - macsam_endog(ssn,'m_sam_tot') ;

 display macsam_endog, macsam_bal_CHK ;
 

$ontext

* Define non-zero macro-sam elements based on aggregated proto-sam
* Include total row and column
* macmnz(ss,ssp)$macsam_endog(ss,ssp) = yes ;

* Non-zero elements in macsam_endog. Will count as non-zero if there are non-zero
* elements in the aggregation, even if the aggregate is zero.
 macmnz(ssn,ssnp) =
  SUM((sac,sacp)$(map_ss_sac(ssn,sac) and map_ss_sac(ssnp,sacp) and P_SAM(sac,sacp)), 1) ;

* Add in total row and column if they have non-zero elements
 MACMNZ("m_sam_tot",ssn) = SUM(ss2np$macsam_endog(ss2np,ssn), 1) ;
 MACMNZ(ssn,"m_sam_tot") = SUM(ss2np$macsam_endog(ssn,ss2np), 1) ;
 
$offtext
 
*~~~~~~~~~~~~~~~~~ END OF DEFINE MACSAM_ENDOG ~~~~~~~~~~~~~~~~~