*##################### 10. INITIALISE VARIABLES AND SET BOUNDS ##########

$onText

This file has 2 parts each with several steps
  1.  Initialize variables
       a. TVs and Coefficients
       b. Macro SAM
       c. Errors and Error weights
       d. Objective function
       
  2.  Define bounds on cell values
       a. Set bounds and fix variables that need to be fixed
       b. Set lower bound of zero on coeffs
       c. Fix coeff to zero if it is in izero
       d. Upper and lower bounds on the error weights
       e. Fix ZERO elements of aggregate-sag AGGSAM  and macro-sam MACSAM

$offText

*Initialize variables

*~~~ TVs and Coefficients
 SAM.L(irow,icol)          = P_SAM0(irow,icol) ;
 COEFF.L(irow,icol)        = coeff0(irow,icol) ;
 COLSUM.L(icol)            = colsum0(icol) ;
 ROWSUM.L(irow)            = rowsum0(irow) ;
 
*~~~ Macro SAM
 MACSAM.L(ss,ssp)          = MACSAM_ENDOG(ss,ssp) ;
 
*~~~ Errors and Error weights
 ERR_RC.L(icol,rwcl)           = 0.0 ;
 ERR_AGG.L(aggr2)              = 0.0 ;
 ERR_TV.L(irow,icol)           = 0.0 ;
 ERR_MSAM.L(ssn,ssnp)          = 0.0 ;

 W_RC.L(icol,rwcl,jwt_RC)      = wbar_RC(icol,rwcl,jwt_RC) ;
 W_AGG.L(aggr2,jwt_AGG)        = wbar_AGG(aggr2,jwt_AGG) ;
 W_TV.L(irow,icol,jwt_TV)      = wbar_TV(irow,icol,jwt_TV) ;
 W_MSAM.L(ss,ssp,jwt_MSAM)     = wbar_MSAM(ss,ssp,jwt_MSAM) ;
 
*~~~ Objective function
 DENTROPY.L                = 0.0 ;

*##################### END OF VARIABLE INITIALISATION ###################

*##################### 11. DEFINE BOUNDS ON CELL VALUES #################

* Set bounds and fix variables that need to be fixed

 SAM.FX(irow,icol)$ifixv(irow,icol)            = P_SAM0(irow,icol) ;

* Set lower bound of zero on coeffs
 COEFF.LO(irow,icol)$(icoeff(irow,icol) and lcell(irow,icol)) = 0.0 ;

* Fix coeff to zero if it is in izero
 COEFF.FX(irow,icol)$izero(irow,icol)  = 0.0 ;

* Fix coeff to zero if it is not a ivalue
* This is not appropriate. COEFF defined in coeffdef equation
* COEFF.FX(sacn, sacnp)$(not ivalue(sacn,sacnp)) = 0.0 ;

*Set lower bound of cutoff on positive cells.
*If cells have additive errors, then it is possible for them to have a zero,
*small, or negative value after adjustment.
*If errors are multiplicative, then lower bound is enforced.
 SAM.LO(irow,icol)$((P_SAM0(irow,icol) gt 0) and lcell(irow,icol)) = cutoff ;

* Upper and lower bounds on the error weights
 W_RC.LO(icol,rwcl,jwt_RC)                            = 0.0   ;
 W_RC.UP(icol,rwcl,jwt_RC)                            = 1.0   ;
 W_AGG.LO(aggr2,jwt_AGG)                              = 0.0   ;
 W_AGG.UP(aggr2,jwt_AGG)                              = 1.0   ;
 W_TV.LO(irow,icol,jwt_TV)                            = 0.0   ;
 W_TV.UP(irow,icol,jwt_TV)                            = 1.0   ;
 W_MSAM.LO(ss,ssp,jwt_MSAM)                           = 0.0   ;
 W_MSAM.UP(ss,ssp,jwt_MSAM)                           = 1.0   ;

*If standard deviation of errors is set to zero, fix the weights to the prior
*so that the error will always be zero.

* Note that it should not be necessary to fix W if ERR is fixed.
* Now exclude equations for errors and error weights when sigmay is zero.

 W_RC.FX(sacn,rwcl,jwt_RC)$(sigmay_RC(sacn,rwcl) eq 0.0)   = wbar_RC(sacn,rwcl,jwt_RC)  ;
 W_AGG.FX(aggr2,jwt_AGG)$(sigmay_AGG(aggr2) eq 0.0)        = wbar_AGG(aggr2,jwt_AGG)     ;
 W_TV.FX(irow,icol,jwt_TV)$(sigmay_TV(irow,icol) eq 0.0)   = wbar_TV(irow,icol,jwt_TV)  ;
 W_MSAM.FX(ss,ssp,jwt_MSAM)$(sigmay_MSAM(ss,ssp) eq 0.0)   = wbar_MSAM(ss,ssp,jwt_MSAM)   ;

 ERR_RC.FX(sacn,rwcl)$(sigmay_RC(sacn,rwcl) eq 0.0)        = 0.0 ;
 ERR_AGG.FX(aggr2)$(sigmay_AGG(aggr2) eq 0.0)              = 0.0 ;
 ERR_TV.FX(irow,icol)$(sigmay_TV(irow,icol) eq 0.0)        = 0.0 ;
 ERR_MSAM.FX(ss,ssp)$(sigmay_MSAM(ss,ssp) eq 0.0)          = 0.0 ;

* Fix elements of macro-sam MACSAM that are always zero.

 MACSAM.FX(ssn,ssnp)$(macmnz(ssn,ssnp) eq 0.0)             = 0.0 ;

*##################### END OF DEFINE BOUNDS FOR CELL VALUES #############